<?php

class briSql {
	private $sql = null;
	private $error = null;
	public $detail = "";
	public $valid = false;
	
	function __construct() {
		include('connection.inc.php');
		if (mysqli_connect_error()) $this->error = mysqli_connect_error();
		else $this->valid = true;
	}
	
	function __destruct() {
		if ($this->sql == null) return;
		try { $this->sql->close(); } catch (Exception $ex) { }
	}
	
	/** returns the last error received by the connection, sql object or the statement object. */
	public function error() {
		if ($this->error) return $this->error;
		else return $this->sql->error;
	}
	
	/** for insert, delete and update statements, add where clause values as additional arguments. return false on failure. */
	public function modify($sql) {
		if ($stmt = $this->sql->prepare($sql)) {
			$binds = array($types);
			$valcount = func_num_args();
			for ($i = 2; $i < $valcount; $i++) {
				$v = func_get_arg($i);
				array_push($binds, $v);
			}
			call_user_func_array(array($stmt, "bind_param"), $binds);
			$res = $stmt->execute();
			$stmt->close();
			return $res;
		} else return false;
	}
	/** for insert, delete and update statements, add where clause values as an array. return false on failure. */
	public function modifyWithArray($sql, $values) {
		if (count($values) == 0) return $this->modifyNoWhere($sql);
		else if ($stmt = $this->sql->prepare($sql)) {
			//get the types of the values into an array
			$valcount = count($values);
			$types = "";
			for ($i = 0; $i < $valcount; $i++) {
				if (is_int($values[$i])) $types .= "i";
				else if (is_float($values[$i])) $types .= "d";
				else $types .= "s";
			}
			
			//create the array with the parameters that will be sent
			$binds = array($types);
			for ($i = 0; $i < $valcount; $i++) {
				$v = $values[$i];
				array_push($binds, $v);
			}
			
			//call bind_param
			call_user_func_array(array($stmt, "bind_param"), $binds);
			
			//execute the sql and return the result
			$res = $stmt->execute();
			$stmt->close();
			return $res;
		} else return false;
	}
	/** for insert, delete and update statements with no where clause */
	public function modifyNoWhere($sql) {
		if ($stmt = $this->sql->prepare($sql)) {
			$res = $stmt->execute();
			$stmt->close();
			return $res;
		} else return false;
	}
	
	/** for select statements, add where clause values as additional arguments */
	public function select($sql) {
		$valcount = func_num_args();
		$values = array();
		for ($i = 1; $i < $valcount; $i++) {
			$v = func_get_arg($i);
			array_push($values, $v);
		}
		return $this->selectWithArray($sql, $values);
	}
	/** for select statements, add where clause values as an array */
	public function selectWithArray($sql, $values) {
		//$this->detail .= ("selectWithArray called with " . $sql . " :: " . implode(", ", $values) . "; ");
		if (count($values) == 0) return $this->selectNoWhere($sql);
		else if ($stmt = $this->sql->prepare($sql)) {
			//get the types of the values into an array
			$valcount = count($values);
			$types = "";
			for ($i = 0; $i < $valcount; $i++) {
				if (is_int($values[$i])) $types .= "i";
				else if (is_float($values[$i])) $types .= "d";
				else $types .= "s";
			}
			
			//create the array with the parameters that will be sent
			$binds = array($types);
			for ($i = 0; $i < $valcount; $i++) {
				$v = $values[$i];
				array_push($binds, $v);
			}
			
			//$this->detail .= ("binds is " . implode(",", $binds) . "; ");
			//call bind_param
			call_user_func_array(array($stmt, "bind_param"), $binds);
			
			//execute the sql
			$res = $stmt->execute();
			//$this->detail .= ("[" . $sql . "]::" . $res);
			
			//parse the results
			$results = null;
			if ($res) {
				$results = $this->fetch($stmt);
			} else $this->error = $stmt->error;
			$stmt->close();
			return $results;
		} else return null;
	}
	/** for select statements, no where clause */
	public function selectNoWhere($sql) {
		if ($stmt = $this->sql->prepare($sql)) {
			$res = $stmt->execute();
			$results = null;
			if ($res) {
				$results = $this->fetch($stmt);
			} else $this->error = $stmt->error;
			$stmt->close();
			return $results;
		} else return null;
	}
	
	/** creates an array holding an assoc-array for each row in the result */
	private function fetch($stmt) {
		$array = array();

		$stmt->store_result();
		$variables = array();
		$data = array();
		$meta = $stmt->result_metadata();

		while($field = $meta->fetch_field()) $variables[] = &$data[$field->name];
		call_user_func_array(array($stmt, 'bind_result'), $variables);

		$i = 0;
		while($stmt->fetch()) {
			$array[$i] = array();
			foreach($data as $k=>$v) $array[$i][$k] = $v;
			$i++;
		}

		return $array;
	}
}

?>
